# FIXME: Use POO for this module like nltk corpus load module
def list_of_corpus():
    return [
        ["axolotl", "Is a Spanish-Nahuatl parallel corpus"],
        ["tsunkua", "Is a Spanish-Otomí parallel corpus"],
        ["kolo", "Is a Spanish-Mixteco parallel corpus"],
    ]


def load(corpus_name):
    # https://stackoverflow.com/questions/6028000/how-to-read-a-static-file-from-inside-a-python-package
    # l1,l2,variant,document_name,*extrafields (depends on corpus)
    available_corpus = ["axolotl", "tsunkua", "kolo"]

    if corpus_name in available_corpus:
        from .. import corpora

        try:
            # For Python >= 3.7
            import importlib.resources as pkg_resources
        except ImportError:
            # Try backported to Python < 3.7 `importlib_resources`.
            import importlib_resources as pkg_resources
        from io import StringIO
        import csv

        corpus_content = pkg_resources.read_text(
            corpora, corpus_name + ".csv", encoding="utf-8", errors="strict"
        )
        corpus_buff = StringIO(corpus_content)
        corpus_reader = csv.reader(corpus_buff)

        return list(corpus_reader)
    else:
        return 0



#
# Información dialectal (ej. cuales códigos ISO manejamos, y cuál es el 
# default).
# 
# TODO: entrenar un modelo general (multi-dialectal) y añadir "all" como una 
# de las opciones.
#
DEFAULT_LANG_CODE = "nhi"
SUPPORTED_LANG_CODES = ["nhi", "nci", "azz"]
DEFAULT_ORTOGRAPHY = "sep"
AVAILABLE_ORTHOGRAPHIES = ["sep", "inali", "ack", "ilv"]


# -*- coding: UTF-8 -*-

"""
Ejemplo de uso:


	>>> from elotl.nahuatl.morphology import Analyser
	>>> a = Analyser()
	>>> res = a.analyse('“Amo quen ximati, teh xiyo in escuela.', tokenise=True)
"""
import logging
from elotl.utils.fst.attapply import ATTFST
from elotl.nahuatl.orthography import Normalizer as Normaliser
import elotl.utils.morphology
from elotl.nahuatl.config import SUPPORTED_LANG_CODES, DEFAULT_LANG_CODE

try:
	# For Python >= 3.7
	import importlib.resources as pkg_resources
except ImportError:
	# Try backported to Python < 3.7 `importlib_resources`.
	import importlib_resources as pkg_resources

logger = logging.getLogger(__name__)


class Analyser(elotl.utils.morphology.Analyser):
	"""
	Class for returning morphological analyses in a Python-friendly format
	with UD-style POS tags and Feature=Value pairs.

	Parameters
	----------
	tokeniser: function
		A tokenisation function, if none is provided a default tokeniser, _tokenise()
		is used which is based on regular expressions.

	"""
	def __init__(self, tokeniser=None, lang_code=None):
		self.tokenise = self._tokenise

		if lang_code is None:
			self.lang_code = DEFAULT_LANG_CODE
			logger.info("No Nahuatl variant language code provided. "
			            "Defaulting to `nhi`.")
		else:
			if lang_code not in SUPPORTED_LANG_CODES:
				logger.error("Unsupported language variant specified.")
				raise ValueError(f"Unsupported lang code for Nahuatl: "
								 f"{lang_code}")
			else:
				self.lang_code = lang_code
		if tokeniser:
			self.tokenise = tokeniser

		_path_to_att_dir = pkg_resources.files("elotl.nahuatl.data").joinpath(f"{self.lang_code}.mor.att")
		_path_to_tsv_dir = pkg_resources.files("elotl.nahuatl.data").joinpath(f"{self.lang_code}.mor.tsv")

		self.analyser = ATTFST(_path_to_att_dir)
		self.convertor = elotl.utils.morphology.Convertor(_path_to_tsv_dir)
		self.normaliser = Normaliser("ack")

# Convenience alias for Analyser to Analyzer
Analyzer = Analyser



# -*- coding: UTF-8 -*-

"""
Ejemplo de uso:

	>>> from elotl.otomi.morphology import Analyser
	>>> a = Analyser()
	>>> res = a.analyse('Mä ga ze̱ngua mä dada habu̱ bí ʼbu̱i', tokenise=True)
"""
import logging
import re
from elotl.utils.fst.attapply import ATTFST
import elotl.utils.morphology
from elotl.otomi.config import SUPPORTED_LANG_CODES, DEFAULT_LANG_CODE

try:
	# For Python >= 3.7
	import importlib.resources as pkg_resources
except ImportError:
	# Try backported to Python < 3.7 `importlib_resources`.
	import importlib_resources as pkg_resources

logger = logging.getLogger(__name__)


class Analyser(elotl.utils.morphology.Analyser):
	"""
	Class for returning morphological analyses in a Python-friendly format
	with UD-style POS tags and Feature=Value pairs.

	Parameters
	----------
	tokeniser: function
		A tokenisation function, if none is provided a default tokeniser, _tokenise()
		is used which is based on regular expressions.

	"""
	def __init__(self, tokeniser=None, lang_code=None):
		self.tokenise = self._tokenise

		if lang_code is None:
			self.lang_code = DEFAULT_LANG_CODE
			logger.info("No Otomi variant language code provided. "
			            "Defaulting to `ote`.")
		else:
			if lang_code not in SUPPORTED_LANG_CODES:
				logger.error("Unsupported language variant specified.")
				logger.error("Available codes: " + 
						" ".join(SUPPORTED_LANG_CODES))
				raise ValueError(f"Unsupported lang code for Otomi: "
								 f"{lang_code}")
			else:
				self.lang_code = lang_code
		if tokeniser:
			self.tokenise = tokeniser

		with pkg_resources.path("elotl.otomi.data", f"{self.lang_code}.mor.att") as p:
			_path_to_att_dir = p
		with pkg_resources.path("elotl.otomi.data", f"{self.lang_code}.mor.tsv") as p:
			_path_to_tsv_dir = p

		self.analyser = ATTFST(_path_to_att_dir)
		self.convertor = elotl.utils.morphology.Convertor(_path_to_tsv_dir)
		self.normaliser = None

	def _tokenise(self, text):
		"""
		Internal backoff tokenisation function.

		Parameters
		----------
		text: str
			The text to be tokenised.

		Returns
		----------
		list
			List of space separated tokens.

		"""

		"""
		This regular expression is a combination of all word symbols 
		and the diacritic symbols mentioned above. Otomi uses
		the U+0331 COMBINING MACRON BELOW symbol, and combining characters
		are usually not included in the set of word symbols.
		"""
		diacritics = '\u0331'
		tokens = re.sub(r'([^\w' + diacritics + '])', r' \g<1> ', text)
		return [token.strip()
			for token in tokens.split(' ')
				if not token.strip() == '']


# Convenience alias for Analyser to Analyzer
Analyzer = Analyser



# -*- coding: UTF-8 -*-

# Para usar desde la línea de comandos:
#     $ python elotl/otomi/orthography.py "<texto>" -ort [inali|otq|ots|rfe]

# O, desde otro programa de Python:

#     >>> from elotl.otomi.orthography import Normalizer
#     >>> normalizer = Normalizer("otq")  # o "ots" "rfe" "inali"
#     >>> normalizer.normalize("<texto>")  # o `normalizer.to_phones("<texto>")`

from __future__ import annotations
import logging
from elotl.utils.fst.attapply import ATTFST
from elotl.otomi.config import DEFAULT_ORTOGRAPHY, AVAILABLE_ORTHOGRAPHIES

# https://docs.python.org/3/library/importlib.html?highlight=resources#module-importlib.resources
try:
    # For Python >= 3.7
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to Python < 3.7 `importlib_resources`.
    import importlib_resources as pkg_resources

# https://importlib-resources.readthedocs.io/en/latest/using.html#migrating-from-legacy
_path_to_orig_fon = pkg_resources.files("elotl.utils.fst.att.otomi").joinpath('orig-fon.att')

_ORIG_FON_FST = ATTFST(_path_to_orig_fon)
logger = logging.getLogger(__name__)


class Normalizer(object):
    """
    Class for normalizing Otomi texts to a single orthography. Currently
    supported output orthographies:
    - OTS (e.g. "kjämadi")
    - OTQ (e.g. "hämadi")
    - INALI (e.g. "jämadi")
    - RFE (e.g. "hąmadi") 

    The entry points for converting text are `.normalize(...)` and
    `.to_phones(...)`.

    Parameters
    ----------
    normalized_ort: str
        Name of the orthography to convert everything into. Must be one of
        ("inali", "otq", "ots", "rfe").
    
    log_level: str
        Desired level of logging ("error", "warn", or "debug"). If "warn" or
        "debug", a message will be produced every time the normalizer is unable
        to convert a word in the input. This can be a bit annoying, so by
        default the log level is set to "error".

    """
    def __init__(self, normalized_ort: str = "inali", log_level="error"):
        if normalized_ort is None:
            normalized_ort = DEFAULT_ORTOGRAPHY
            logger.info(f"No Otomi Ortography code provided. Defaulting to {DEFAULT_ORTOGRAPHY}")

        if not (normalized_ort in AVAILABLE_ORTHOGRAPHIES):
            logger.warning(normalized_ort + " is not a supported orthography.")
            logger.warning(f"Using '{DEFAULT_ORTOGRAPHY}' as orthography.")
            normalized_ort = DEFAULT_ORTOGRAPHY

        log_level = log_level.lower()
        if log_level == "warn":
            logger.setLevel(logging.WARN)
        elif log_level == "error":
            logger.setLevel(logging.ERROR)
        elif log_level == "debug":
            logger.setLevel(logging.DEBUG)
        else:
            logger.setLevel(logging.ERROR)
            logging.error("Log level '{}' not recognized. Setting log level to"
                          " 'ERROR'.".format(log_level))

        with pkg_resources.path("elotl.utils.fst.att.otomi",
                                "fon-" + normalized_ort + ".att") as p:
            _path_to_att_dir = p

        self.norm_fst = ATTFST(_path_to_att_dir)

    def _convert(self, w: str, fst: ATTFST) -> str:
        """
        Convert an input word form to an output form using the provided ATTFST
        object. In this implementation, we assume high weights are preferred,
        so we select the last of the generated candidates.

        Parameters
        ----------
        w: str
            Input word.

        fst: ATTFST
            FST object created with attapply. This object has an `apply`
            command used to apply the fst to an input.

        Returns
        -------
        str
            The generated string with the highest weight.

        """
        w = w.lower()
        forms = list(fst.apply(w))
        if forms:
            return forms[-1][0]

    def _g2p(self, w):
        """
        Converts an input word to a sequence of phonemes using an FST defined
        in elotl/otomi/fst/lexc/orig-fon.lexc. FIX: THIS COMMENT IS WRONG (SAME IN NAHUATL/ORTOGRAHPY)
        """
        return self._convert(w, _ORIG_FON_FST)

    def _normalize_word(self, original_word: str) -> tuple[str, str]:
        """
        Convert an input word from 'any' orthography into a normalized
        orthography (currently OTQ, INALI, OTS, and RFE). Since this process
        requires first converting the input to a pseudo-phonemic
        representation, we return both the phonemic and normalized forms.

        Parameters
        ----------
        original_word: str
            Input word form, in theory in any mixture of common Otomi
            orthographies.

        Returns
        -------
        Tuple
            Phonemic, Normalized forms of the string.

        """
        w = original_word.lower()

        fon = self._g2p(w)
        if fon is None:
            logger.warn("Unable to convert word '{}' to phonemes."
                        .format(w))
            return w, w

        normed = self._convert(fon, self.norm_fst)
        if normed is None:
            logger.warn("Unable to convert word '{}'.from phonemes to "
                        "normalized orthography.".format(fon))
            return fon, w

        return fon, normed

    @staticmethod
    def _tokenize(s):
        return s.split()

    def to_phones(self, text: str, overrides: dict[str, str] = None) -> str:
        """
        Convert a non-normalized Otomi text into approximate/pseudo IPA. 
        Conversion happens at the word-level after tokenizing on whitespace.

        Parameters
        ----------
        text: str
            Input text. It can be from any of a number of possible Otomi
            orthographies (the system attempts to handle many of the graphic
            variations observed in diverse Otomi texts).

        overrides: dict
            A dictionary of hard-coded normalizations. If an input word
            (lowered) is contained in the dictionary, we will use the form it
            maps to instead of applying the FST on it.

        Returns
        -------
        str
            Whitespace-joined words converted to approximate phonemic
            representation.

        """
        overrides = overrides if overrides is not None else {}
        fon = []
        for token in self._tokenize(text):
            if token in overrides:
                t_fon = overrides[token.lower()]
            t_fon = self._g2p(token)
            if t_fon is None:
                logger.warn("Unable to convert word '{}' to phonemes."
                            .format(token))
                fon.append(token)
                continue
            fon.append(t_fon)

        return " ".join(fon)

    def normalize(self, text: str, overrides: dict[str, str] = None) -> str:
        """
        Convert a non-normalized Otomi text into normalized orthography.
        Depending on the value used when initializing this class, the
        normalized orthography is OTQ, INALI, OTS, and RFE. Conversion happens
        at the word-level after tokenizing on whitespace.

        Parameters
        ----------
        text: str
            Input text. It can be from any of a number of possible Otomi
            orthographies (the system attempts to handle many of the graphic
            variations observed in diverse Otomi texts).

        overrides: dict
            A dictionary of hard-coded normalizations. If an input word
            (lowered) is contained in the dictionary, we will use the form it
            maps to instead of applying the FST on it.

        Returns
        -------
        str
            Whitespace-joined words converted to normalized orthography.

        """
        overrides = overrides if overrides is not None else {}
        norm = []
        for token in self._tokenize(text):
            if token in overrides:
                norm.append(overrides[token])
                continue
            _, t_norm = self._normalize_word(token)

            norm.append(t_norm)

        return " ".join(norm)



# -*- coding: UTF-8 -*-

"""
Ejemplo de uso:

	>>> from elotl.nahuatl.morphology import Analyser
	>>> a = Analyser()
	>>> res = a.analyse('“Amo quen ximati, teh xiyo in escuela.', tokenise=True)
"""

import re
from elotl.utils.fst.attapply import ATTFST

try:
	# For Python >= 3.7
	import importlib.resources as pkg_resources
except ImportError:
	# Try backported to Python < 3.7 `importlib_resources`.
	import importlib_resources as pkg_resources


class Token(object):
	"""
	Each analysis is formed of a weight and a sequence of words, a token may
	contain more than one word, a word is a dictionary of
		{lemma, pos, {feat:value}}
	"""
	def __init__(self, wordform, analyses=[], max_analyses=None):
		self.wordform = wordform
		self.analyses = analyses[:max_analyses]
		self.pos = None
		self.lemma = None

		"""
		If the lemma and POS are unambiguous (e.g. all analyses share the
		same lemma and POS then we can set the global lemma/POS to that.
		For analyses that contain more than one syntactic word, e.g.
		contractions, this is not possible.
		"""
		lemmas = set()
		categories = set()
		for (analysis, weight) in self.analyses:
			if len(analysis) == 1:
				lemmas.add(analysis[0]['lemma'])
				categories.add(analysis[0]['pos'])
		if len(lemmas) == 1:
			self.lemma = lemmas.pop()
		if len(categories) == 1:
			self.pos = categories.pop()

	def __repr__(self):
		return '<Token form="{0}" lemma="{1}" pos="{2}" analyses="{3}">'.format(
			self.wordform, self.lemma, self.pos, len(self.analyses))


class Convertor(object):
	"""
	A class to convert from Apertium-style analyses to UD-compatible analyses by
	means of a sequence of priority-ordered rules.

	Parameters
	----------
	rule_file: str
		A path to the location of the TSV file containing the tagset
		conversion rules.
		Each rule is composed of 9 columns:
			1. Priority
			2. Input lemma
			3. Input POS
			4. Input features
			5. Input dependency relation (unused)
			6. Output lemma
			7. Output POS
			8. Output features
			9. Output dependency relation (unused)

	"""
	def __init__(self, rule_file):
		self.conversion_rules = self._load_conversion_rules(rule_file)
		self.input_patterns = re.compile('(' + '|'.join(self.conversion_rules['sym']) + ')')

	def _convert_tags(self, tags):
		"""
		Convert from the tag format in the TSV rule-file to tags that will match
		the Apertium analyses, e.g. v|tv → <v><tv> and p1|sg → <p1><sg>

		Parameters
		----------
		tags: str
			A sequence of tags separated by the pipe symbol, |

		Returns
		----------
		str
			A sequences of tags encased in less-than and greater-than < >
		"""

		if tags != '':
			return '<' + tags.replace('|', '><') + '>'
		return tags

	def _load_conversion_rules(self, fn):
		"""
		Loads the conversion rules and scores each rule. Longer rules are
		scored higher, rules containing lemmas are scored higher.

		Parameters
		----------
		fn: str
			A path to the location of the TSV file containing the
			tagset conversion rules.

		Returns
		----------
		dict
			A dictionary containing a set of symbols used for matching and a list of
			substitution rules.
		"""

		rules = {'sym': set(), 'sub': []}
		with open(fn, "r", encoding='UTF-8') as file:
			for line in file:
				if line[0] == '#':
					continue
				row = line.strip().split('\t')
				score = int(row[0])
				inn = [re.sub('^_$', '', i) for i in row[1:5]]
				out = [re.sub('^_$', '', i) for i in row[5:]]

				if inn[0] != '':
					score += 4
				if inn[1] != '':
					score += 3
				if inn[2] != '':
					score += (2 * len(inn[2].split('|')))
				if inn[3] != '':
					score += 1

				inn = [inn[0],
						self._convert_tags(inn[1]), self._convert_tags(inn[2]), inn[3]]
				rules['sym'].add(inn[1])
				rules['sym'].add(inn[2])
				rules['sub'].append((score, inn, out))

		rules['sym'] = list(rules['sym'])
		rules['sym'].sort(key=lambda x: len(x), reverse=True)
		rules['sub'].sort()
		return rules

	def _convert(self, a):
		"""
		Convert an analysis to UD using the conversion rules, rules
		are applied in priority order.

		Parameters
		----------
		a: str
			An Apertium-compatible analysis, e.g. <s_sg1>quiza<v><iv><pret>, note
			that by this point there should be no subwords.

		Returns
		----------
		dict
			A dictionary containing a lemma, a part-of-speech and a
			set of Feature=Value pairs.
		"""
		analysis = {'lemma': '', 'pos': '', 'feats': set()}
		tags = [i for i in self.input_patterns.findall(a) if not i == '']
		msd = set(tags)
		analysis['lemma'] = re.sub('<[^>]+>', '', a)
		for (priority, inn, out) in self.conversion_rules['sub']:
			pos_pat = set([inn[1]])
			remainder = msd - pos_pat
			intersect = msd.intersection(pos_pat)
			if intersect == pos_pat:
				analysis['pos'] = out[1]
				if out[2] != '':
					for i in out[2].split('|'):
						analysis['feats'].add(i)
				msd = remainder
			feats_pat = set([inn[2]])
			remainder = msd - feats_pat
			intersect = msd.intersection(feats_pat)
			if intersect == feats_pat:
				for i in out[2].split('|'):
					analysis['feats'].add(i)
				msd = remainder

		# Convert set of Feature=Value pairs to dictionary of Feature:Value
		analysis['feats'] = {i.split('=')[0]: i.split('=')[1]
					for i in analysis['feats'] if not i == ''}

		return analysis

	def convert(self, analysis_):
		"""
		The main function for conversion, takes a full analysis, including possible
		subwords, e.g. ya<adv>+<s_sg1>quiza<v><iv><pret> and returns a list of
		syntactic words.

		Parameters
		----------
		analysis_: str
			An Apertium-compatible analysis.

		Returns
		----------
		list
			A list of the component analyses by syntactic word.
		"""

		analysis = []
		subwords = analysis_.split('+')
		for word in subwords:
			analysis.append(self._convert(word))
		return analysis


class Analyser(object):
	"""
	Class for returning morphological analyses in a Python-friendly format
	with UD-style POS tags and Feature=Value pairs.

	Parameters
	----------
	tokeniser: function
		A tokenisation function, if none is provided a default tokeniser, _tokenise()
		is used which is based on regular expressions.

	"""
	def __init__(self, tokeniser=None):
		self.tokenise = self._tokenise

		if tokeniser:
			self.tokenise = tokeniser

		with pkg_resources.path("elotl.nahuatl.data", "nhi.mor.att") as p:
			_path_to_att_dir = p
		with pkg_resources.path("elotl.nahuatl.data", "nhi.mor.tsv") as p:
			_path_to_tsv_dir = p

		self.analyser = ATTFST(_path_to_att_dir)
		self.convertor = Convertor(_path_to_tsv_dir)

	def _tokenise(self, text):
		"""
		Internal backoff tokenisation function.

		Parameters
		----------
		text: str
			The text to be tokenised.

		Returns
		----------
		list
			List of space separated tokens.

		"""

		tokens = re.sub(r'([^\w])', r' \g<1> ', text)
		return [token.strip()
			for token in tokens.split(' ')
				if not token.strip() == '']

	def _convert_analysis(self, analysis):
		"""
		Takes an analysis returned by the transducer and converts it
		to a more Python-friendly format.

		Parameters
		----------
		analysis: str
			An analysis for a surface token, e.g. ya<adv>+<s_sg1>quiza<v><iv><pret>

		Returns
		----------
		list
			A list of syntactic words with their analyses in UD format
			[{'lemma': 'ya', 'pos': 'ADV', 'feats': set()},
				{'lemma': 'quiza', 'pos': 'VERB', 'feats': {
				'Number[subj]=Sing', 'Person[subj]=1', 'Tense=Past', i
					'VerbForm=Fin', 'Subcat=Intr'}}]
		"""

		analyses = self.convertor.convert(analysis)
		return analyses

	def _analyse_token(self, token, alternatives=[]):
		"""
		Function that takes a token and returns a list of analyses with their
		weights as assigned by the transducer.

		Parameters
		----------
		token: str
			A surface token

		alternative: list
			A list of alternative forms, e.g. a lowercased or normalised form

		Returns
		----------
		list
			A list of the possible analyses with weights
		"""

		# Look up the word form in the FST
		analyses = list(self.analyser.apply(token))
		i = 0
		# If the result is empty, try cycling through all the alternatives
		while analyses == []:
			if i >= len(alternatives):
				break
			analyses = list(self.analyser.apply(alternatives[i]))
			i += 1

		# Sort the analyses by weight, higher weight = worse
		analyses.sort(key=lambda x: x[1])

		converted = []
		for (analysis, weight) in analyses:
			converted.append((self._convert_analysis(analysis), weight))

		return converted

	def analyse(self, text, tokenise=False, normalise=False, max_analyses=None):
		"""
		An analyse function that can take either a string with a tokeniser,
		a pre-tokenised list or a pre-tokenised string. If it is passed a
		tokeniser the tokeniser is first run and then the list is analysed,
		if passed a list, the list is analysed, if passed a pre-tokenised
		list then the list is split on space and then analysed.

		Parameters
		----------
		text: str or list
			Input text/sentence. It can be either a string or a list. It
			should be pre-tokenised, alternatively see below.

		tokenise: bool
			Should tokenisation be performed on the input prior to analysis?

		normalise: bool
			Should normalised forms be added to the list of alternatives?

		max_analyses: int
			The maximum number of analyses that should be returned, these
			will be returned in order of weight, lowest weight first. If
			None, return all analyses.

		Returns
		----------
		list
			List of Token objects.

		Examples
		----------

		>>> a.analyse('a b c')
		[<Token "a" (0)>, <Token "b" (0)>, <Token "c" (0)>]
		>>> a.analyse(['a', 'b', 'c'])
		[<Token "a" (0)>, <Token "b" (0)>, <Token "c" (0)>]
		>>> a.analyse('ab, c', tokenise=True)
		[<Token "a" (0)>, <Token "b" (0)>, <Token "," (0)>, <Token "c" (0)>]
		"""

		tokens = []
		wordforms = []
		if not tokenise:
			if type(text) == str:
				wordforms = text.split(' ')
			elif type(text) == list:
				wordforms = text
		else:
			wordforms = self.tokenise(text)

		for wordform in wordforms:
			alternatives = [wordform.lower()]
			if normalise and self.normaliser:
				alternatives += [self.normaliser.normalize(wordform),
					self.normaliser.normalize(wordform).lower()]
			analyses = self._analyse_token(wordform, alternatives)
			token = Token(wordform, analyses, max_analyses)
			tokens.append(token)

		return tokens

	def analyze(self, text, tokenize=False, normalize=False, max_analyses=None):
		"""
		Convenience alias of analyse() with alternative spelling.

		Parameters
		----------
		text: str or list
			Input text/sentence. It can be either a string or a list. It
			should be pre-tokenised, alternatively see below.

		tokenize: bool
			Should tokenisation be performed on the input prior to analysis?

		max_analyses: int
			The maximum number of analyses that should be returned, these
			will be returned in order of weight, lowest weight first. If
			None, return all analyses.

		Returns
		----------
		list
			List of Token objects.
		"""
		return self.analyse(text, tokenize, normalize, max_analyses)

# Convenience alias for Analyser to Analyzer
Analyzer = Analyser



# -*- coding: UTF-8 -*-

"""
Ejemplo de uso:


	>>> from elotl.huave.morphology import Analyser
	>>> a = Analyser()
	>>> res = a.analyse('Teat tepood tambas mal wiiüd sawün win.', tokenise=True)
"""

from elotl.utils.fst.attapply import ATTFST
#from elotl.huave.orthography import Normalizer as Normaliser
import elotl.utils.morphology

try:
	# For Python >= 3.7
	import importlib.resources as pkg_resources
except ImportError:
	# Try backported to Python < 3.7 `importlib_resources`.
	import importlib_resources as pkg_resources

class Analyser(elotl.utils.morphology.Analyser):
	"""
	Class for returning morphological analyses in a Python-friendly format
	with UD-style POS tags and Feature=Value pairs.

	Parameters
	----------
	tokeniser: function
		A tokenisation function, if none is provided a default tokeniser, _tokenise()
		is used which is based on regular expressions.

	"""
	def __init__(self, tokeniser=None):
		self.tokenise = self._tokenise

		if tokeniser:
			self.tokenise = tokeniser

		with pkg_resources.path("elotl.huave.data", "huv.mor.att") as p:
			_path_to_att_dir = p
		with pkg_resources.path("elotl.huave.data", "huv.mor.tsv") as p:
			_path_to_tsv_dir = p

		self.analyser = ATTFST(_path_to_att_dir)
		self.convertor = elotl.utils.morphology.Convertor(_path_to_tsv_dir)
		#self.normaliser = Normaliser("ack")
		self.normaliser = None

# Convenience alias for Analyser to Analyzer
Analyzer = Analyser
